@@ -16,7 +16,7 @@ package Clownfish::Build::Binding;
 use strict;
 use warnings;
 
-our $VERSION = '0.004099_004';
+our $VERSION = '0.005000';
 $VERSION = eval $VERSION;
 
 sub bind_all {
@@ -200,6 +200,7 @@ singleton(either_sv, value)
     bool     value;
 CODE:
 {
+    CFISH_UNUSED_VAR(either_sv);
     RETVAL = CFISH_OBJ_TO_SV_INC(cfish_Bool_singleton(value));
 }
 OUTPUT: RETVAL
