@@ -23,8 +23,9 @@
 #include "Clownfish/Test/TestString.h"
 
 #include "Clownfish/String.h"
+#include "Clownfish/Boolean.h"
+#include "Clownfish/ByteBuf.h"
 #include "Clownfish/CharBuf.h"
-#include "Clownfish/Num.h"
 #include "Clownfish/Test.h"
 #include "Clownfish/TestHarness/TestBatchRunner.h"
 #include "Clownfish/TestHarness/TestUtils.h"
@@ -51,9 +52,9 @@ static String*
 S_smiley_with_whitespace(int *num_spaces_ptr) {
     int32_t spaces[] = {
         ' ',    '\t',   '\r',   '\n',   0x000B, 0x000C, 0x000D, 0x0085,
-        0x00A0, 0x1680, 0x180E, 0x2000, 0x2001, 0x2002, 0x2003, 0x2004,
-        0x2005, 0x2006, 0x2007, 0x2008, 0x2009, 0x200A, 0x2028, 0x2029,
-        0x202F, 0x205F, 0x3000
+        0x00A0, 0x1680, 0x2000, 0x2001, 0x2002, 0x2003, 0x2004, 0x2005,
+        0x2006, 0x2007, 0x2008, 0x2009, 0x200A, 0x2028, 0x2029, 0x202F,
+        0x205F, 0x3000
     };
     int num_spaces = sizeof(spaces) / sizeof(uint32_t);
 
@@ -69,6 +70,51 @@ S_smiley_with_whitespace(int *num_spaces_ptr) {
     return retval;
 }
 
+static void
+test_new(TestBatchRunner *runner) {
+    static char chars[] = "A string " SMILEY " with a smile.";
+
+    {
+        char *buffer = (char*)MALLOCATE(sizeof(chars));
+        strcpy(buffer, chars);
+        String *thief = Str_new_steal_utf8(buffer, sizeof(chars) - 1);
+        TEST_TRUE(runner, Str_Equals_Utf8(thief, chars, sizeof(chars) - 1),
+                  "Str_new_steal_utf8");
+        DECREF(thief);
+    }
+
+    {
+        char *buffer = (char*)MALLOCATE(sizeof(chars));
+        strcpy(buffer, chars);
+        String *thief
+            = Str_new_steal_trusted_utf8(buffer, sizeof(chars) - 1);
+        TEST_TRUE(runner, Str_Equals_Utf8(thief, chars, sizeof(chars) - 1),
+                  "Str_new_steal_trusted_utf8");
+        DECREF(thief);
+    }
+
+    {
+        String *wrapper = Str_new_wrap_utf8(chars, sizeof(chars) - 1);
+        TEST_TRUE(runner, Str_Equals_Utf8(wrapper, chars, sizeof(chars) - 1),
+                  "Str_new_wrap_utf8");
+        DECREF(wrapper);
+    }
+
+    {
+        String *wrapper = Str_new_wrap_trusted_utf8(chars, sizeof(chars) - 1);
+        TEST_TRUE(runner, Str_Equals_Utf8(wrapper, chars, sizeof(chars) - 1),
+                  "Str_new_wrap_trusted_utf8");
+        DECREF(wrapper);
+    }
+
+    {
+        String *smiley_str = Str_new_from_char(smiley_cp);
+        TEST_TRUE(runner, Str_Equals_Utf8(smiley_str, smiley, smiley_len),
+                  "Str_new_from_char");
+        DECREF(smiley_str);
+    }
+}
+
 static void
 test_Cat(TestBatchRunner *runner) {
     String *wanted = Str_newf("a%s", smiley);
@@ -105,28 +151,45 @@ test_Clone(TestBatchRunner *runner) {
     DECREF(wanted);
 }
 
+static int64_t
+S_find(String *string, String *substring) {
+    StringIterator *iter = Str_Find(string, substring);
+    if (iter == NULL) { return -1; }
+    size_t tick = StrIter_Recede(iter, SIZE_MAX);
+    DECREF(iter);
+    return (int64_t)tick;
+}
+
 static void
-test_Find(TestBatchRunner *runner) {
+test_Contains_and_Find(TestBatchRunner *runner) {
     String *string;
     String *substring = S_get_str("foo");
 
     string = S_get_str("");
-    TEST_TRUE(runner, Str_Find(string, substring) == -1, "Not in empty string");
+    TEST_FALSE(runner, Str_Contains(string, substring),
+               "Not contained in empty string");
+    TEST_INT_EQ(runner, S_find(string, substring), -1,
+                "Not found in empty string");
     DECREF(string);
 
     string = S_get_str("foo");
-    TEST_TRUE(runner, Str_Find(string, substring) == 0, "Find complete string");
+    TEST_TRUE(runner, Str_Contains(string, substring),
+              "Contained in complete string");
+    TEST_INT_EQ(runner, S_find(string, substring), 0, "Find complete string");
     DECREF(string);
 
     string = S_get_str("afoo");
-    TEST_TRUE(runner, Str_Find(string, substring) == 1, "Find after first");
-    // TODO: Enable this test when we have real substrings.
-    /*Str_Set_Size(string, 3);
-    TEST_TRUE(runner, Str_Find(string, substring) == -1, "Don't overrun");*/
+    TEST_TRUE(runner, Str_Contains(string, substring),
+              "Contained after first");
+    TEST_INT_EQ(runner, S_find(string, substring), 1, "Find after first");
+    String *prefix = Str_SubString(string, 0, 3);
+    TEST_FALSE(runner, Str_Contains(prefix, substring), "Don't overrun");
+    DECREF(prefix);
     DECREF(string);
 
     string = S_get_str("afood");
-    TEST_TRUE(runner, Str_Find(string, substring) == 1, "Find in middle");
+    TEST_TRUE(runner, Str_Contains(string, substring), "Contained in middle");
+    TEST_INT_EQ(runner, S_find(string, substring), 1, "Find in middle");
     DECREF(string);
 
     DECREF(substring);
@@ -149,18 +212,39 @@ test_Code_Point_At_and_From(TestBatchRunner *runner) {
                     code_points[i], "Code_Point_From %ld", (long)from);
     }
 
+    TEST_INT_EQ(runner, Str_Code_Point_At(string, num_code_points), STR_OOB,
+                "Code_Point_At %ld", (long)num_code_points);
+    TEST_INT_EQ(runner, Str_Code_Point_From(string, 0), STR_OOB,
+                "Code_Point_From 0");
+    TEST_INT_EQ(runner, Str_Code_Point_From(string, num_code_points + 1),
+                STR_OOB, "Code_Point_From %ld", (long)(num_code_points + 1));
+
     DECREF(string);
 }
 
 static void
 test_SubString(TestBatchRunner *runner) {
-    String *string = Str_newf("a%s%sb%sc", smiley, smiley, smiley);
-    String *wanted = Str_newf("%sb%s", smiley, smiley);
-    String *got = Str_SubString(string, 2, 3);
-    TEST_TRUE(runner, Str_Equals(wanted, (Obj*)got), "SubString");
-    DECREF(wanted);
-    DECREF(got);
-    DECREF(string);
+    {
+        String *string = Str_newf("a%s%sb%sc", smiley, smiley, smiley);
+        String *wanted = Str_newf("%sb%s", smiley, smiley);
+        String *got = Str_SubString(string, 2, 3);
+        TEST_TRUE(runner, Str_Equals(wanted, (Obj*)got), "SubString");
+        DECREF(string);
+        DECREF(wanted);
+        DECREF(got);
+    }
+
+    {
+        static const char chars[] = "A string.";
+        String *wrapper = Str_new_wrap_utf8(chars, sizeof(chars) - 1);
+        String *wanted  = Str_newf("string");
+        String *got     = Str_SubString(wrapper, 2, 6);
+        TEST_TRUE(runner, Str_Equals(got, (Obj*)wanted),
+                  "SubString with wrapped buffer");
+        DECREF(wrapper);
+        DECREF(wanted);
+        DECREF(got);
+    }
 }
 
 static void
@@ -255,6 +339,29 @@ test_To_I64(TestBatchRunner *runner) {
     string = S_get_str("-10");
     TEST_TRUE(runner, Str_To_I64(string) == -10, "To_I64 negative");
     DECREF(string);
+
+    string = S_get_str("10.");
+    TEST_TRUE(runner, Str_To_I64(string) == 10, "To_I64 stops at non-digits");
+    DECREF(string);
+
+    string = S_get_str("10A");
+    TEST_TRUE(runner, Str_To_I64(string) == 10,
+              "To_I64 stops at out-of-range digits");
+    DECREF(string);
+
+    string = S_get_str("-JJ");
+    TEST_TRUE(runner, Str_BaseX_To_I64(string, 20) == -399,
+              "BaseX_To_I64 base 20");
+    DECREF(string);
+}
+
+static void
+test_To_String(TestBatchRunner *runner) {
+    String *string = Str_newf("Test");
+    String *copy   = Str_To_String(string);
+    TEST_TRUE(runner, Str_Equals(copy, (Obj*)string), "To_String");
+    DECREF(string);
+    DECREF(copy);
 }
 
 static void
@@ -267,6 +374,15 @@ test_To_Utf8(TestBatchRunner *runner) {
     DECREF(string);
 }
 
+static void
+test_To_ByteBuf(TestBatchRunner *runner) {
+    String     *string = Str_newf("foo");
+    ByteBuf    *bb     = Str_To_ByteBuf(string);
+    TEST_TRUE(runner, BB_Equals_Bytes(bb, "foo", 3), "To_ByteBuf");
+    DECREF(bb);
+    DECREF(string);
+}
+
 static void
 test_Length(TestBatchRunner *runner) {
     String *string = Str_newf("a%s%sb%sc", smiley, smiley, smiley);
@@ -297,14 +413,61 @@ test_Compare_To(TestBatchRunner *runner) {
 }
 
 static void
-test_Swap_Chars(TestBatchRunner *runner) {
-    String *source = S_get_str("aXXbXc");
-    String *got    = Str_Swap_Chars(source, 'X', smiley_cp);
-    String *wanted = Str_newf("a%s%sb%sc", smiley, smiley, smiley);
-    TEST_TRUE(runner, Str_Equals(got, (Obj*)wanted), "Swap_Chars");
-    DECREF(wanted);
-    DECREF(got);
-    DECREF(source);
+test_Starts_Ends_With(TestBatchRunner *runner) {
+    String *prefix = S_get_str("pre" SMILEY "fix_");
+    String *suffix = S_get_str("_post" SMILEY "fix");
+    String *empty  = S_get_str("");
+
+    TEST_TRUE(runner, Str_Starts_With(suffix, suffix),
+              "Starts_With self returns true");
+    TEST_TRUE(runner, Str_Starts_With(prefix, prefix),
+              "Ends_With self returns true");
+
+    TEST_TRUE(runner, Str_Starts_With(suffix, empty),
+              "Starts_With empty string returns true");
+    TEST_TRUE(runner, Str_Ends_With(prefix, empty),
+              "Ends_With empty string returns true");
+    TEST_FALSE(runner, Str_Starts_With(empty, suffix),
+              "Empty string Starts_With returns false");
+    TEST_FALSE(runner, Str_Ends_With(empty, prefix),
+              "Empty string Ends_With returns false");
+
+    {
+        String *string
+            = S_get_str("pre" SMILEY "fix_string_post" SMILEY "fix");
+        TEST_TRUE(runner, Str_Starts_With(string, prefix),
+                  "Starts_With returns true");
+        TEST_TRUE(runner, Str_Ends_With(string, suffix),
+                  "Ends_With returns true");
+        DECREF(string);
+    }
+
+    {
+        String *string
+            = S_get_str("pre" SMILEY "fix:string:post" SMILEY "fix");
+        TEST_FALSE(runner, Str_Starts_With(string, prefix),
+                   "Starts_With returns false");
+        TEST_FALSE(runner, Str_Ends_With(string, suffix),
+                   "Ends_With returns false");
+        DECREF(string);
+    }
+
+    DECREF(prefix);
+    DECREF(suffix);
+    DECREF(empty);
+}
+
+static void
+test_Get_Ptr8(TestBatchRunner *runner) {
+    String *string = S_get_str("Banana");
+
+    const char *ptr8 = Str_Get_Ptr8(string);
+    TEST_TRUE(runner, strcmp(ptr8, "Banana") == 0, "Get_Ptr8");
+
+    size_t size = Str_Get_Size(string);
+    TEST_INT_EQ(runner, size, 6, "Get_Size");
+
+    DECREF(string);
 }
 
 static void
@@ -328,6 +491,17 @@ test_iterator(TestBatchRunner *runner) {
     }
     String *string = CB_To_String(buf);
 
+    {
+        StringIterator *iter = Str_Top(string);
+
+        TEST_TRUE(runner, StrIter_Equals(iter, (Obj*)iter),
+                  "StringIterator equal to self");
+        TEST_FALSE(runner, StrIter_Equals(iter, (Obj*)CFISH_TRUE),
+                   "StringIterator not equal non-iterators");
+
+        DECREF(iter);
+    }
+
     {
         StringIterator *top  = Str_Top(string);
         StringIterator *tail = Str_Tail(string);
@@ -345,6 +519,16 @@ test_iterator(TestBatchRunner *runner) {
         StrIter_Assign(clone, tail);
         TEST_TRUE(runner, StrIter_Equals(clone, (Obj*)tail), "Assign");
 
+        String *other = Str_newf("Other string");
+        StringIterator *other_iter = Str_Top(other);
+        TEST_FALSE(runner, StrIter_Equals(other_iter, (Obj*)tail),
+                   "Equals returns false for different strings");
+        StrIter_Assign(clone, other_iter);
+        TEST_TRUE(runner, StrIter_Equals(clone, (Obj*)other_iter),
+                  "Assign iterator with different string");
+
+        DECREF(other);
+        DECREF(other_iter);
         DECREF(clone);
         DECREF(top);
         DECREF(tail);
@@ -361,7 +545,7 @@ test_iterator(TestBatchRunner *runner) {
 
         TEST_TRUE(runner, !StrIter_Has_Next(iter),
                   "Has_Next at end of string");
-        TEST_INT_EQ(runner, StrIter_Next(iter), STRITER_DONE,
+        TEST_INT_EQ(runner, StrIter_Next(iter), STR_OOB,
                     "Next at end of string");
 
         StringIterator *tail = Str_Tail(string);
@@ -374,7 +558,7 @@ test_iterator(TestBatchRunner *runner) {
     {
         StringIterator *iter = Str_Tail(string);
 
-        for (int i = num_code_points - 1; i >= 0; --i) {
+        for (size_t i = num_code_points; i--;) {
             TEST_TRUE(runner, StrIter_Has_Prev(iter), "Has_Prev %d", i);
             int32_t code_point = StrIter_Prev(iter);
             TEST_INT_EQ(runner, code_point, code_points[i], "Prev %d", i);
@@ -382,7 +566,7 @@ test_iterator(TestBatchRunner *runner) {
 
         TEST_TRUE(runner, !StrIter_Has_Prev(iter),
                   "Has_Prev at end of string");
-        TEST_INT_EQ(runner, StrIter_Prev(iter), STRITER_DONE,
+        TEST_INT_EQ(runner, StrIter_Prev(iter), STR_OOB,
                     "Prev at start of string");
 
         StringIterator *top = Str_Top(string);
@@ -426,19 +610,19 @@ test_iterator_whitespace(TestBatchRunner *runner) {
 
     {
         StringIterator *iter = Str_Top(ws_smiley);
-        TEST_INT_EQ(runner, StrIter_Skip_Next_Whitespace(iter), num_spaces,
-                    "Skip_Next_Whitespace");
-        TEST_INT_EQ(runner, StrIter_Skip_Next_Whitespace(iter), 0,
-                    "Skip_Next_Whitespace without whitespace");
+        TEST_INT_EQ(runner, StrIter_Skip_Whitespace(iter), num_spaces,
+                    "Skip_Whitespace");
+        TEST_INT_EQ(runner, StrIter_Skip_Whitespace(iter), 0,
+                    "Skip_Whitespace without whitespace");
         DECREF(iter);
     }
 
     {
         StringIterator *iter = Str_Tail(ws_smiley);
-        TEST_INT_EQ(runner, StrIter_Skip_Prev_Whitespace(iter), num_spaces,
-                    "Skip_Prev_Whitespace");
-        TEST_INT_EQ(runner, StrIter_Skip_Prev_Whitespace(iter), 0,
-                    "Skip_Prev_Whitespace without whitespace");
+        TEST_INT_EQ(runner, StrIter_Skip_Whitespace_Back(iter), num_spaces,
+                    "Skip_Whitespace_Back");
+        TEST_INT_EQ(runner, StrIter_Skip_Whitespace_Back(iter), 0,
+                    "Skip_Whitespace_Back without whitespace");
         DECREF(iter);
     }
 
@@ -453,9 +637,9 @@ test_iterator_substring(TestBatchRunner *runner) {
     StringIterator *end = Str_Tail(string);
 
     {
-        String *substring = StrIter_substring(start, end);
+        String *substring = StrIter_crop(start, end);
         TEST_TRUE(runner, Str_Equals(substring, (Obj*)string),
-                  "StrIter_substring whole string");
+                  "StrIter_crop whole string");
         DECREF(substring);
     }
 
@@ -463,32 +647,51 @@ test_iterator_substring(TestBatchRunner *runner) {
     StrIter_Recede(end, 2);
 
     {
-        String *substring = StrIter_substring(start, end);
+        String *substring = StrIter_crop(start, end);
         String *wanted = Str_newf("b%sc", smiley);
         TEST_TRUE(runner, Str_Equals(substring, (Obj*)wanted),
-                  "StrIter_substring");
+                  "StrIter_crop");
 
-        TEST_TRUE(runner, StrIter_Starts_With(start, wanted), "Starts_With");
-        TEST_TRUE(runner, StrIter_Ends_With(end, wanted), "Ends_With");
+        TEST_TRUE(runner, StrIter_Starts_With(start, wanted),
+                  "Starts_With returns true");
+        TEST_TRUE(runner, StrIter_Ends_With(end, wanted),
+                  "Ends_With returns true");
 
         DECREF(wanted);
         DECREF(substring);
     }
 
     {
-        String *substring = StrIter_substring(end, NULL);
+        String *short_str = Str_newf("b%sx", smiley);
+        TEST_FALSE(runner, StrIter_Starts_With(start, short_str),
+                   "Starts_With returns false");
+        TEST_FALSE(runner, StrIter_Ends_With(start, short_str),
+                   "Ends_With returns false");
+
+        String *long_str = Str_newf("b%sxxxxxxxxxxxx%sc", smiley, smiley);
+        TEST_FALSE(runner, StrIter_Starts_With(start, long_str),
+                   "Starts_With long string returns false");
+        TEST_FALSE(runner, StrIter_Ends_With(end, long_str),
+                   "Ends_With long string returns false");
+
+        DECREF(short_str);
+        DECREF(long_str);
+    }
+
+    {
+        String *substring = StrIter_crop(end, NULL);
         String *wanted = Str_newf("%sd", smiley);
         TEST_TRUE(runner, Str_Equals(substring, (Obj*)wanted),
-                  "StrIter_substring with NULL tail");
+                  "StrIter_crop with NULL tail");
         DECREF(wanted);
         DECREF(substring);
     }
 
     {
-        String *substring = StrIter_substring(NULL, start);
+        String *substring = StrIter_crop(NULL, start);
         String *wanted = Str_newf("a%s", smiley);
         TEST_TRUE(runner, Str_Equals(substring, (Obj*)wanted),
-                  "StrIter_substring with NULL top");
+                  "StrIter_crop with NULL top");
         DECREF(wanted);
         DECREF(substring);
     }
@@ -500,22 +703,34 @@ test_iterator_substring(TestBatchRunner *runner) {
 
 void
 TestStr_Run_IMP(TestString *self, TestBatchRunner *runner) {
-    TestBatchRunner_Plan(runner, (TestBatch*)self, 101);
+    TestBatchRunner_Plan(runner, (TestBatch*)self, 139);
+    test_new(runner);
     test_Cat(runner);
     test_Clone(runner);
     test_Code_Point_At_and_From(runner);
-    test_Find(runner);
+    test_Contains_and_Find(runner);
     test_SubString(runner);
     test_Trim(runner);
     test_To_F64(runner);
     test_To_I64(runner);
+    test_To_String(runner);
     test_To_Utf8(runner);
+    test_To_ByteBuf(runner);
     test_Length(runner);
     test_Compare_To(runner);
-    test_Swap_Chars(runner);
+    test_Starts_Ends_With(runner);
+    test_Get_Ptr8(runner);
     test_iterator(runner);
     test_iterator_whitespace(runner);
     test_iterator_substring(runner);
 }
 
+/*************************** StringCallbackTest ***************************/
+
+bool
+StrCbTest_Unchanged_By_Callback_IMP(StringCallbackTest *self, String *str) {
+    String *before = Str_Clone(str);
+    StrCbTest_Callback(self);
+    return Str_Equals(str, (Obj*)before);
+}
 
